package net.sf.itcb.addons.cachemanager;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import net.sf.ehcache.CacheException;
import net.sf.ehcache.CacheManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import generated.Ehcache;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.oxm.XmlMappingException;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;


/**
 * Class based on spring's EhCacheManagerFactoryBean class.
 * Its aim is to merge ehcache configs getting them in jar files
 * @author Pierre Le Roux
 */
public class ItcbEhCacheManagerFactoryBean implements FactoryBean<CacheManager>, InitializingBean, DisposableBean {

	protected Resource[] configLocations;
	
	private Jaxb2Marshaller ehcacheMarshaller;

	
	protected final Log logger = LogFactory.getLog(getClass());

	private boolean shared = false;

	private String cacheManagerName;

	private CacheManager cacheManager;


	/**
	 * Set the location of the EHCache config files.
	 * The config files included in itcb-cache-manager imports all files that match the pattern :
	 * classpath*:WEB-INF/*ehcache.xml
	 * Wild card is allowed. 
	 * ConfigLocations XML files are merged.
	 * @throws IOException 
	 * @see net.sf.ehcache.CacheManager#create(java.io.InputStream)
	 * @see net.sf.ehcache.CacheManager#CacheManager(java.io.InputStream)
	 */
	public void setConfigLocations(String configLocations) throws IOException {
		PathMatchingResourcePatternResolver c = new PathMatchingResourcePatternResolver();
		this.configLocations = c.getResources(configLocations);
	}
	/**
	 * Set whether the EHCache CacheManager should be shared (as a singleton at the VM level)
	 * or independent (typically local within the application). Default is "false", creating
	 * an independent instance.
	 * @see net.sf.ehcache.CacheManager#create()
	 * @see net.sf.ehcache.CacheManager#CacheManager()
	 */
	public void setShared(boolean shared) {
		this.shared = shared;
	}

	/**
	 * Set the name of the EHCache CacheManager (if a specific name is desired).
	 * @see net.sf.ehcache.CacheManager#setName(String)
	 */
	public void setCacheManagerName(String cacheManagerName) {
		this.cacheManagerName = cacheManagerName;
	}
	
	/**
	 * Set EhCacheMarshaller
	 * @throws IOException
	 * @throws CacheException
	 */
	public void setEhcacheMarshaller(Jaxb2Marshaller ehcacheMarshaller) {
		this.ehcacheMarshaller = ehcacheMarshaller;
	}


	public void afterPropertiesSet() throws IOException, CacheException {
		logger.info("Initializing EHCache CacheManager");
		if (this.shared) {
			// Shared CacheManager singleton at the VM level.
			if (this.configLocations != null) {
				this.cacheManager = CacheManager.create(getInputStream());
			}
			else {
				this.cacheManager = CacheManager.create();
			}
		}
		else {
			// Independent CacheManager instance (the default).
			if (this.configLocations != null) {
				this.cacheManager = new CacheManager(getInputStream());
			}
			else {
				this.cacheManager = new CacheManager();
			}
		}
		if (this.cacheManagerName != null) {
			this.cacheManager.setName(this.cacheManagerName);
		}
	}


	/**
	 * This method merge ehcache xml configuration files as an inputstream
	 * @return the inputStream
	 * @throws XmlMappingException
	 * @throws IOException
	 */
	private InputStream getInputStream() throws XmlMappingException, IOException {
		Ehcache ehcacheToReturn=null;
		for (Resource ehcacheXml : this.configLocations) {
			logger.debug(ehcacheXml.getURI().toString());
			Ehcache ehcache;
			if(ehcacheToReturn==null) {
				ehcacheToReturn = (Ehcache)ehcacheMarshaller.unmarshal(new StreamSource(ehcacheXml.getInputStream()));
			}
			else {
				ehcache= (Ehcache)ehcacheMarshaller.unmarshal(new StreamSource(ehcacheXml.getInputStream()));
				ehcacheToReturn.getCache().addAll(ehcache.getCache());
			}
		}
		
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		StreamResult stream = new StreamResult(bos);
		ehcacheMarshaller.marshal(ehcacheToReturn, stream);
		return new ByteArrayInputStream(bos.toByteArray());
	}
	
	public CacheManager getObject() {
		return this.cacheManager;
	}

	public Class<? extends CacheManager> getObjectType() {
		return (this.cacheManager != null ? this.cacheManager.getClass() : CacheManager.class);
	}

	public boolean isSingleton() {
		return true;
	}


	public void destroy() {
		logger.info("Shutting down EHCache CacheManager");
		this.cacheManager.shutdown();
	}

	
}

